"""Tests for Modat Collector

SPDX-FileCopyrightText: 2025 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
import unittest
from unittest import mock

import intelmq.lib.message as message
import requests
from requests_mock import MockerCore

from intelmq_extensions.bots.collectors.modat.collector import ModatCollectorBot

from ....base import BotTestCase


class TestModatCollectorBot(BotTestCase, unittest.TestCase):
    @classmethod
    def set_bot(cls):
        cls.bot_reference = ModatCollectorBot
        cls.sysconfig = {
            "query": "test-query",
            "type": "host",
            "api_key": "secure-api-key",
            "code": "feed-code",
        }

    def mock_http_session(self):
        session = requests.Session()
        session_mock = mock.patch(
            "intelmq_extensions.bots.collectors.modat.collector.create_request_session",
            return_value=session,
        )
        session_mock.start()
        self.addCleanup(session_mock.stop)

        self.requests = MockerCore(session=session)
        self.requests.start()
        self.addCleanup(self.requests.stop)

    def setUp(self):
        super().setUp()
        self.mock_http_session()

    def mock_request(
        self,
        path: str,
        expected_query: str = "test-query",
        expected_page: int = 1,
        expected_page_size: int = 10,
        **kwargs,
    ):
        def check_request(request):
            if request.headers.get("Authorization", "") != "Bearer secure-api-key":
                return False
            if request.json()["query"] != expected_query:
                return False
            if request.json()["page"] != expected_page:
                return False
            if request.json()["page_size"] != expected_page_size:
                return False
            return True

        self.requests.post(
            f"https://api.magnify.modat.io/{path}",
            status_code=200,
            additional_matcher=check_request,
            **kwargs,
        )

    def _create_report_dict(self, data, **kwargs):
        report = message.Report(kwargs, harmonization=self.harmonization)
        report.add("feed.name", "Test Bot")
        report.add("feed.accuracy", 100.0)
        report.add("feed.code", "feed-code")
        report.add("raw", json.dumps(data))
        return report.to_dict(with_type=True)

    def test_query_hosts(self):
        self.mock_request(
            "host/search/v1",
            json={
                "page_nr": 1,
                "total_pages": 1,
                "total_records": 4,
                "page": [{"ip": "ip1"}, {"ip": "ip2"}],
            },
        )
        self.mock_request(
            "host/search/v1",
            json={
                "page_nr": 2,
                "total_pages": 1,
                "total_records": 4,
                "page": [{"ip": "ip3"}, {"ip": "ip4"}],
            },
            expected_page=2,
        )

        self.run_bot()

        self.assertOutputQueueLen(2)
        self.assertMessageEqual(
            0, self._create_report_dict([{"ip": "ip1"}, {"ip": "ip2"}])
        )
        self.assertMessageEqual(
            1, self._create_report_dict([{"ip": "ip3"}, {"ip": "ip4"}])
        )
        self.assertEqual(2, self.requests.call_count)



"""Data for tests for ModatParserBot

SPDX-FileCopyrightText: 2025 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

RESPONSE_1 = [
    {
        "ip": "127.0.0.1",
        "geo": {
            "city_name": "Vienna",
            "country_name": "Austria",
            "country_iso_code": "AT",
        },
        "asn": {"number": 8412, "org": "T-Mobile Austria GmbH"},
        "fqdns": ["domain.example.at", "device-123.example.at"],
        "is_anycast": False,
        "tags": [],
        "cves": [
            {"id": "CVE-2020-11985", "cvss": 5.3, "is_kev": False},
            {"id": "CVE-2025-3891", "cvss": 7.5, "is_kev": False},
        ],
        "services": [
            {
                "transport": "tcp",
                "port": 8443,
                "ports": [8443],
                "last_scanned_port": 8443,
                "protocol": "http",
                "scanned_at": "2025-12-17T03:57:49Z",
                "is_match": False,
            },
            {
                "transport": "tcp",
                "port": 49592,
                "ports": [49592],
                "last_scanned_port": 49592,
                "protocol": "http",
                "scanned_at": "2025-05-29T20:24:10.308508Z",
                "is_match": True,
            },
            {
                "transport": "tcp",
                "port": 39895,
                "ports": [39895],
                "last_scanned_port": 39895,
                "protocol": "unknown",
                "scanned_at": "2025-11-22T12:22:14Z",
                "is_match": True,
            },
        ],
    },
    {
        "ip": "127.0.0.2",
        "geo": {
            "city_name": "Feldkirch",
            "country_name": "Austria",
            "country_iso_code": "AT",
        },
        "asn": {"number": 5385, "org": "Russmedia IT GmbH"},
        "fqdns": [],
        "is_anycast": False,
        "tags": [],
        "cves": [
            {"id": "CVE-2016-1247", "cvss": 7.8, "is_kev": False},
            {"id": "CVE-2017-20005", "cvss": 9.8, "is_kev": False},
        ],
        "services": [
            {
                "transport": "tcp",
                "port": 44302,
                "ports": [44302],
                "last_scanned_port": 44302,
                "protocol": "unknown",
                "scanned_at": "2025-11-24T00:30:40Z",
                "is_match": True,
            },
            {
                "transport": "tcp",
                "port": 443,
                "ports": [443],
                "last_scanned_port": 443,
                "protocol": "http",
                "scanned_at": "2025-12-15T15:54:24Z",
                "is_match": False,
            },
            {
                "transport": "tcp",
                "port": 8443,
                "ports": [8443],
                "last_scanned_port": 8443,
                "protocol": "http",
                "scanned_at": "2025-12-16T22:49:47Z",
                "is_match": False,
            },
        ],
    },
]

EVENT_1 = {
    # "raw": RESPONSE1[0],
    "source.ip": "127.0.0.1",
    "source.geolocation.cc": "AT",
    "source.geolocation.country": "Austria",
    "source.geolocation.city": "Vienna",
    "extra": {
        "services": RESPONSE_1[0]["services"],
        "fqdns": "domain.example.at;device-123.example.at",
    },
    "product.vulnerabilities": "cve-2020-11985;cve-2025-3891",
    "protocol.application": "http",
    "protocol.transport": "tcp",
    "source.as_name": "T-Mobile Austria GmbH",
    "source.asn": 8412,
    "source.fqdn": "domain.example.at",
    "source.port": 49592,
    "time.source": "2025-05-29T20:24:10.308508+00:00",
}

EVENT_2 = {
    # "raw": RESPONSE1[1],
    "source.ip": "127.0.0.2",
    "source.geolocation.cc": "AT",
    "source.geolocation.country": "Austria",
    "source.geolocation.city": "Feldkirch",
    "extra": {
        "services": RESPONSE_1[1]["services"],
    },
    "product.vulnerabilities": "cve-2016-1247;cve-2017-20005",
    "protocol.transport": "tcp",
    "source.as_name": "Russmedia IT GmbH",
    "source.asn": 5385,
    "source.port": 44302,
    "time.source": "2025-11-24T00:30:40+00:00",
}



"""Collector of data from Modat API

SPDX-FileCopyrightText: 2025 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later

https://api.magnify.modat.io/docs

Parameters:

api_key
# TODO: multiple queries in one bot?
query
type [service|host]

# with defaults
url = "https://api.magnify.modat.io/"
page_size = 10
max_results = 100

# Standard Collector parameters
name: Optional[str] = None
accuracy: int = 100
code: Optional[str] = None
provider: Optional[str] = None
documentation: Optional[str] = None
"""

import json
from urllib.parse import urljoin

from intelmq.lib.bot import CollectorBot
from intelmq.lib.utils import create_request_session


class ModatCollectorBot(CollectorBot):
    url: str = "https://api.magnify.modat.io/"
    page_size = 10
    max_results = 100

    api_key: str
    query: str
    type: str  # service|host

    def init(self):
        self.set_request_parameters()
        self.session = create_request_session(self)
        self.session.headers = {"Authorization": f"Bearer {self.api_key}"}

    def process(self):
        page = 1
        collected_results = 0
        total_available = self.page_size

        if self.type == "host":
            url = urljoin(self.url, "/host/search/v1")
        else:
            url = urljoin(self.url, "/service/search/v1")

        while (
            total_available > collected_results and collected_results < self.max_results
        ):
            result = self.session.post(
                url,
                json={"page": page, "page_size": self.page_size, "query": self.query},
            )
            if result.status_code != 200:
                self.logger.error("Modat responded with error %d.", result.status_code)
                self.logger.debug("Modat response: %s.", result.text)
                raise RuntimeError(
                    "Cannot retrieve data from Modat.", detail=result.text
                )

            data = result.json()
            report = self.new_report()
            report.add("raw", json.dumps(data["page"]))
            self.send_message(report)

            page += 1
            collected_results += len(data["page"])
            total_available = data["total_records"]

    # @staticmethod
    # def check(parameters: dict) -> list[list[str]] or None:
    #     pass


BOT = ModatCollectorBot



"""Tests for ModatParserBot

SPDX-FileCopyrightText: 2025 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
import unittest
from datetime import datetime

import intelmq.lib.message as message
from dateutil.tz import UTC

from intelmq_extensions.bots.parsers.modat.parser import ModatParserBot

from ....base import BotTestCase
from . import data


class TestModatParserBot(BotTestCase, unittest.TestCase):
    @classmethod
    def set_bot(cls):
        cls.bot_reference = ModatParserBot
        cls.sysconfig = {
            "default_fields": {
                "classification.taxonomy": "other",
                "classification.type": "other",
                "classification.identifier": "something",
            }
        }

    def _create_report_dict(self, raw, **kwargs):
        data = {
            "feed.name": "Test Bot",
            "feed.accuracy": 100.0,
            "feed.code": "feed-code",
            "time.observation": datetime(2023, 1, 1, tzinfo=UTC).isoformat(),
        }
        data.update(kwargs)
        report = message.Report(data, harmonization=self.harmonization)
        report.add("raw", json.dumps(raw))
        return report.to_dict(with_type=True)

    def _create_event_dict(
        self,
        extra: dict,
        # raw: dict,
        **kwargs,
    ):
        data = {
            "classification.identifier": "something",
            "classification.taxonomy": "other",
            "classification.type": "other",
            "time.observation": datetime(2023, 1, 1, tzinfo=UTC).isoformat(),
            "feed.accuracy": 100.0,
            "feed.code": "feed-code",
            "feed.name": "Test Bot",
        }
        data.update(kwargs)
        event = message.Event(data, harmonization=self.harmonization)
        event.add("extra", extra)
        # event.add(
        #     "raw", json.dumps(raw)
        # )
        return event.to_dict(with_type=True)

    def test_parsing_report_default(self):
        self.input_message = self._create_report_dict(data.RESPONSE_1)

        self.run_bot()

        self.assertMessageEqual(
            0,
            self._create_event_dict(
                **data.EVENT_1,
            ),
            compare_raw=False,
        )
        self.assertMessageEqual(
            1,
            self._create_event_dict(**data.EVENT_2),
            compare_raw=False,
        )



"""Parsing Modat search requests

Currently supporting only host search results

SPDX-FileCopyrightText: 2025 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
from datetime import datetime, timezone

import intelmq.lib.message as message
from intelmq.lib import utils
from intelmq.lib.bot import ParserBot


class ModatParserBot(ParserBot):
    def parse(self, report: message.Report):
        raw_report = utils.base64_decode(report.get("raw"))
        report_data = json.loads(raw_report)

        for entry in report_data:
            self._current_line = json.dumps(entry)
            yield entry

    def parse_line(self, line: dict, report: message.Report):
        event = self.new_event(report)
        event.add("raw", self._current_line)

        event.add("source.ip", line.get("ip"), raise_failure=False)
        event.add(
            "source.geolocation.cc",
            line.get("geo", {}).get("country_iso_code"),
            raise_failure=False,
        )
        event.add(
            "source.geolocation.country",
            line.get("geo", {}).get("country_name"),
            raise_failure=False,
        )
        event.add(
            "source.geolocation.city",
            line.get("geo", {}).get("city_name"),
            raise_failure=False,
        )

        event.add("source.asn", line.get("asn", {}).get("number"), raise_failure=False)
        event.add("source.as_name", line.get("asn", {}).get("org"), raise_failure=False)
        fqdns = line.get("fqdns", [])
        if fqdns:
            event.add("source.fqdn", fqdns[0], raise_failure=False)
        if len(fqdns) > 1:
            event.add("extra.fqdns", ";".join(fqdns), raise_failure=False)
        event.add("extra.tag", ";".join(line.get("tags", [])), raise_failure=False)

        cves = ";".join(cve.get("id", "").lower() for cve in line.get("cves", []))
        if cves:
            event.add("product.vulnerabilities", cves, raise_failure=False)

        services = line.get("services", [])
        last_scanned = datetime.now(tz=timezone.utc)
        if services:
            last_scanned = max(s["scanned_at"] for s in services)
            event.add("extra.services", services, raise_failure=False)

            for service in services:
                # This is what we were looking for
                if service["is_match"]:
                    event.add(
                        "protocol.application",
                        service.get("protocol"),
                        ignore=(None, "unknown"),
                        raise_failure=False,
                    )
                    event.add(
                        "protocol.transport",
                        service.get("transport"),
                        raise_failure=False,
                    )
                    event.add("source.port", service.get("port"), raise_failure=False)

                    last_scanned = service.get("scanned_at") or last_scanned

                    # TODO: emit more events?
                    break

        event.add("time.source", last_scanned, raise_failure=False)

        return event


BOT = ModatParserBot
